#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
Author: xuwei
Email: weix@sensedeal.ai
Description:
'''

import re


class Parse(object):
    def __init__(self):
        pass

    # 经过排除正则后返回为真继续处理
    @classmethod
    def __exclude_parse(cls, txt, exclude):
        if type(txt).__name__ != 'str':
            return False
        # 去除含有某些字段的句子
        if exclude == '':
            return True
        out = re.compile(exclude)
        rs = out.search(txt)
        if rs:
            return False
        else:
            return True

    @classmethod
    def match_label(cls, txt, contain, label='match'):
        # 含有某些字段的句子
        p = re.compile(contain)
        r = p.search(txt)
        if r:
            return label
        else:
            return None

    # 逗号代表and，包含规则和排除规则都可以有
    @classmethod
    def __and_exclude(cls, txt, exclude):
        if ',' in exclude:
            _exclude_list = exclude.split(',')
            _res_list = []
            for _exclude in _exclude_list:
                _res_list.append(cls.__exclude_parse(txt, _exclude))
            if len(set(_res_list)) == 1 and _res_list[0] == False:
                return False
        else:
            _remove_res = cls.__exclude_parse(txt, exclude)
            if _remove_res == False:
                return False
        return True

    # 逗号代表and，包含规则和排除规则都可以有
    @classmethod
    def __and_contain(cls, txt, contain, label):
        if ',' in contain:
            _contain_list = contain.split(',')
            for _contain in _contain_list:
                _res = cls.match_label(txt, _contain, label)
                if _res == None:
                    return None
            return label
        else:
            _res = cls.match_label(txt, contain, label)
            return _res

    @classmethod
    def re_match_label(cls, txt, contain, exclude='', label='match'):
        """
        :param txt:
        :param contain: 包含正则
        :param exclude: 排除正则
        :param label: txt标签
        :return:
        """
        if exclude:
            # 分号代表排除规则有一个没通过则不通过
            _exclude_list = exclude.split(';')
            for _exclude in _exclude_list:
                _exclude_state = cls.__and_exclude(txt, _exclude)
                if not _exclude_state:
                    return None

        _contain_list = contain.split(';')
        for _contain in _contain_list:
            _match_res = cls.__and_contain(txt, _contain, label)
            if _match_res:
                return _match_res
        return None

    @classmethod
    def re_find_all(cls, txt, contain):
        # 含有某些字段的句子
        p = re.compile(contain)
        r = p.findall(txt)
        return r

    # 逗号代表and，包含规则和排除规则都可以有
    @classmethod
    def __round_find_all(cls, txt, contain, exclude='', sep_and=','):
        """
        :param txt:
        :param contain: 包含正则
        :param exclude: 排除正则
        :param label: txt标签
        :return:
        """
        if sep_and in exclude:
            _exclude_list = exclude.split(',')
            _res_list = []
            for _exclude in _exclude_list:
                _res_list.append(cls.__exclude_parse(txt, _exclude))
            if len(set(_res_list)) == 1 and _res_list[0] == False:
                return []
        else:
            _remove_res = cls.__exclude_parse(txt, exclude)
            if _remove_res == False:
                return []

        if sep_and in contain:
            _contain_list = contain.split(',')
            _res_list = []
            for _contain in _contain_list:
                _list = cls.re_find_all(txt, _contain)
                if not _list:
                    return []
                _res_list += _list
            return _res_list
        else:
            _res_list = cls.re_find_all(txt, contain)
            return _res_list

    @classmethod
    def re_replace(cls, txt, replaced_regexp, new_str):
        match_res_list = cls.re_find_all(txt, replaced_regexp)
        for mm in match_res_list:
            txt = txt.replace(mm, mm.replace(mm, new_str))
        return txt

    @classmethod
    def get_start_end_list(cls, word, txt):
        pos_list = [i.start() for i in re.finditer(word, txt)]
        res_list = []
        for start_pos in pos_list:
            end_pos = start_pos + len(word)
            res_list.append({'start': start_pos, 'end': end_pos})
        return res_list



if __name__ == '__main__':
    _txt = "一季度增长6.4%，二季度中国GDP增速或将下滑到6.3%！那美国呢？"
    _contain = '季度|美国'
    _exclude = ''
    r = Parse.re_find_all(_txt, _contain)
    # r = Parse.re_match_label('徐威喜欢数据文件', '徐|喜欢,文件')
    print(r)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
File: .py
Author: xuwei
Email: weix@sensedeal.ai
Last modified: date
Description:
'''

import datetime
import time


class TimeEtl(object):

    @classmethod
    def ge_now(cls):
        return datetime.datetime.now()

    @classmethod
    def time_str(cls, _now):
        _str_now = _now.strftime("%Y-%m-%d %H:%M:%S")
        return _str_now

    @classmethod
    def time_date(cls, _now):
        _str_now = _now.strftime("%Y-%m-%d")
        return _str_now

    @classmethod
    def time_clock(cls, _now):
        _str_now = _now.strftime("%H:%M:%S")
        return _str_now

    @classmethod
    def time_stamp(cls, _now):
        un_time = time.mktime(_now.timetuple())
        return int(un_time)

    @classmethod
    def str_time(cls, _str):
        _time = datetime.datetime.strptime(_str, "%Y-%m-%d %H:%M:%S")
        return _time

    @classmethod
    def str_time_stamp(cls, _str):
        _time = cls.str_time(_str)
        _time_stamp = cls.time_stamp(_time)
        return _time_stamp

    @classmethod
    def date_time(cls, _str):
        _time = datetime.datetime.strptime(_str, "%Y-%m-%d")
        return _time

    @classmethod
    def stamp_time(cls, _stamp):
        _dt = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(_stamp))
        return _dt

    @classmethod
    def before_day(cls, _now, _day):
        _time = (_now - datetime.timedelta(days=_day)).strftime("%Y-%m-%d")
        return _time

    @classmethod
    def after_day(cls, _now, _day):
        _time = (_now + datetime.timedelta(days=_day)).strftime("%Y-%m-%d")
        return _time


etl_time = TimeEtl()

if __name__ == '__main__':
    _now = datetime.datetime.now()
    _str_time = TimeEtl.time_stamp(_now)
    # print(_str_time)
    #
    _time = TimeEtl.stamp_time(_str_time)
    print(_time)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                          
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved                                                      
Author: xuwei                                        
Email: weix@sensedeal.ai                                 
Description:                                    
'''

from concurrent.futures import ThreadPoolExecutor, wait, ALL_COMPLETED, FIRST_COMPLETED,as_completed
from multiprocessing import Pool
import multiprocessing as mp
from .config_log import log



"""
Python3.2开始，标准库为我们提供了concurrent.futures模块，它提供了ThreadPoolExecutor和ProcessPoolExecutor两个类，
实现了对threading和multiprocessing的进一步抽象，对编写线程池/进程池提供了直接的支持。
"""


def multi_process_map(work_fun, itr_arg, pool_num=3, if_return=False):
    pool = Pool(pool_num)
    res = pool.map(work_fun, itr_arg)
    pool.close()  # 关闭进程池
    pool.join()  # 等待子进程结束
    if if_return:
        return res


# 函数只接受一个参数，然后得到所有函数的返回结果
def multi_process_by_pool(work_fun, itr_arg, pool_num=3, timeout=None):
    res_list = []
    pool = Pool(pool_num)
    all_tasks = [pool.apply_async(work_fun, args=(arg, )) for arg in itr_arg]
    for task in all_tasks:
        try:
            res_list.append(task.get(timeout=timeout))
        except:
            res_list.append(None)
    return res_list


"""
一定要使用with，而不要使用for，如果你一定要用for，那么一定要手动进行executor.shutdown，而你使用了with方法的话，
    在with方法内部已经实现了wait(),在使用完毕之后可以自行关闭线程池，减少资源浪费。

如果不用with：
    executor = ThreadPoolExecutor(max_workers=thread_num)
    all_task = [executor.submit(run_main, (each)) for each in var_list]
    executor.shutdown(wait=True) 等待所有线程执行完毕，相当于进程池的pool.close()+pool.join()操作，wait=True，等待池内所有任务执行完毕回收完资源后才继续，
    wait=False，立即返回，并不会等待池内的任务执行完毕，但不管wait参数为何值，整个程序都会等到所有任务执行完毕，默认True。

    如果不用shutdown，还可以用wait方法让主线程阻塞，直到满足设定的要求。
    wait(all_task, return_when=ALL_COMPLETED) wait方法接收3个参数，等待的任务序列、超时时间以及等待条件。
    等待条件return_when默认为ALL_COMPLETED，表明要等待所有的任务都结束。可以看到运行结果中，确实是所有任务都完成了，
    主线程才打印出main。等待条件还可以设置为FIRST_COMPLETED，表示第一个任务完成就停止等待。
"""


def multi_thread_map(work_fun, itr_arg, thread_num, if_return=False, timeout=None):
    ###多线程
    with ThreadPoolExecutor(max_workers=thread_num) as executor:
        res = executor.map(work_fun, itr_arg, timeout=timeout)
    if if_return:
        return list(res)


# 多线程返回控制，并返回线程值
def multi_thread_return(work_fun, itr_arg, thread_num=3, timeout=None, return_when=FIRST_COMPLETED, if_wait=False):
    res_list = []
    executor = ThreadPoolExecutor(max_workers=thread_num)
    all_tasks = [executor.submit(work_fun, args) for args in itr_arg]
    wait(all_tasks, return_when=return_when)
    executor.shutdown(wait=if_wait)
    for future in all_tasks:
        try:
            res_list.append(future.result(timeout=timeout))
        except:
            res_list.append(None)
    return res_list


# 多线程返回控制，并返回线程值
def multi_thread_return_funs_list(work_funs_list, thread_num=3, timeout=None, return_when=FIRST_COMPLETED, if_wait=False):
    res_list = []
    executor = ThreadPoolExecutor(max_workers=thread_num)
    all_tasks = [executor.submit(work_fun, ) for work_fun in work_funs_list]
    wait(all_tasks, return_when=return_when)
    executor.shutdown(wait=if_wait)
    for future in all_tasks:
        try:
            res_list.append(future.result(timeout=timeout))
        except Exception as e:
            log.exception(e)
            res_list.append(None)
    return res_list


def multi_process(func, args):
    p = mp.Process(target=func, args=args)
    p.start()  # 子进程 开始执行
    p.join()  # 等待子进程结束
    pass


def fun(a):
    print(sum(a))
    return sum(a)


if __name__ == '__main__':
    pass
    current_labeled = multi_process_map(fun, ([0, 1], [2, 3], [4]), 4, if_return=True)
    data = multi_thread_map(fun, ([0, 1], [2, 3], [4]), 2, if_return=True)
    print(data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Author: xuwei
Email: 18810079020@163.com
File: test_data.py
Date: 2020/5/14 2:39 下午
'''

from pyhive import hive
import pandas as pd

class HiveOp(object):
    def __init__(self, host=None, port=None, username=None, password=None, database='default', hive_config=None):
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.hive_config = hive_config

    def get_con(self):
        conn = hive.Connection(host=self.host, database=self.database, port=self.port, username=self.username,
                               password=self.password, configuration=self.hive_config)
        return conn

    def get_dict_sql(self, sql):
        res_list = []
        try:
            conn = self.get_con()
            cursor = conn.cursor()
            cursor.execute(sql)
            query_list = cursor.fetchall()
            columns = cursor.description
            col_names = []
            for column in columns:
                col_names.append(column[0].split('.')[-1])
            # print(col_names)
            for item in query_list:
                res_dict = dict()
                for i in range(len(col_names)):
                    res_dict[col_names[i]] = item[i]
                res_list.append(res_dict)
            return res_list
        except Exception as e:
            print('get_dict_sql error %s' % e)
            raise e
        finally:
            cursor.close()
            conn.close()

    def get_tuple_sql(self, sql):
        try:
            conn = self.get_con()
            cursor = conn.cursor()
            cursor.execute(sql)
            query_list = cursor.fetchall()
            return query_list
        except Exception as e:
            print('get_tuple_sql error %s' % e)
        finally:
            cursor.close()
            conn.close()


    def get_loop_df(self, sql, chunksize=10):
        df_loop = pd.read_sql(sql, self.get_con(), chunksize=chunksize)
        return df_loop



if __name__ == '__main__':
    hive_config = {
        'mapreduce.job.queuename': 'my_hive',
        'hive.exec.compress.output': 'false',
        'hive.exec.compress.intermediate': 'true',
        'mapred.min.split.size.per.node': '1',
        'mapred.min.split.size.per.rack': '1',
        'hive.map.aggr': 'true',
        'hive.groupby.skewindata': 'true'
    }

    hive_op = HiveOp(host='xxxx', database='huanghe_dim', hive_config=hive_config)

    sql = 'SELECT * FROM `huanghe_dim`.`dim_comment_f` LIMIT 100'

    # res = hive_op.get_dict_sql(sql)
    # print(res)
    #
    # print(hive_op.get_tuple_sql(sql))

    df_loop = hive_op.get_loop_df(sql)
    for df in df_loop:
        print(df)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
File: {name}.py
Author: xuwei
Email: weix@sensedeal.ai
Last modified: 2018.12.23
Description:
'''

import json
import time
import numpy
import pymysql
import datetime
import pandas as pd
from datetime import date
from .config_log import config
from multiprocessing import Pool
from sqlalchemy import create_engine
from DBUtils.PooledDB import PooledDB, SharedDBConnection
from DBUtils.PersistentDB import PersistentDB, PersistentDBError, NotSupportedError



# from common.config_log import config

_ProcessNum = 3
_Thread = 4


class DataTypeToStr(object):
    @classmethod
    def data_type_to_str(cls, obj):
        if obj != obj:
            return None
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, numpy.datetime64):
            return pd.to_datetime(str(obj)).strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, numpy.integer) or isinstance(obj, numpy.floating) or isinstance(obj, int) \
                or isinstance(obj, float) or isinstance(obj, bytes):
            return str(obj)
        elif isinstance(obj, numpy.ndarray):
            return json.dumps(obj.tolist(), ensure_ascii=False)
        elif isinstance(obj, list):
            return json.dumps(obj, ensure_ascii=False)
        elif isinstance(obj, dict):
            return json.dumps(obj, ensure_ascii=False)
        else:
            return obj

    @classmethod
    def dict_values_type_etl_str(cls, dict_param):
        for key in dict_param:
            dict_param[key] = cls.data_type_to_str(dict_param[key])
        return dict_param

    @classmethod
    def list_dict_values_type_to_str(cls, list_dict):
        list_dict = list(map(cls.dict_values_type_etl_str, list_dict))
        return list_dict


def engine_con(label, host='', port=0, user='', password='', db='', **kwargs):
    if label:
        _dict = dict(host=config(label, 'host'), port=int(config(label, 'port')), user=config(label, 'user'),
                     passwd=config(label, 'pass'), db=config(label, 'db'))
    else:
        _dict = dict(host=host, port=port, user=user, passwd=password, db=db)
    # print(_dict)
    engine = create_engine("mysql+pymysql://{user}:{passwd}@{host}:{port}/{db}". \
        format(
        host=_dict['host'],
        port=_dict['port'],
        user=_dict['user'],
        passwd=_dict['passwd'],
        db=_dict['db'],
    ),
        connect_args={'charset': 'utf8'},
        pool_size=5,
        pool_recycle=1800,
    )
    return engine


def get_db_con(label, con_type='get_dict', is_multi_thread=False,
               maxconnections=10, mincached=3, maxcached=5, maxshared=0, maxusage=1000,
               host='', port=0, user='', password='', db=''):
    """
    :param label:
    :param con_type: get_dict, get_tuple, get_con
    :param is_multi_thread:
    :param maxconnections:
    :param mincached:
    :param maxcached:
    :param maxshared:
    :param maxusage:
    :return:
    """
    if label:
        _dict = dict(host=config(label, 'host'), port=int(config(label, 'port')), user=config(label, 'user'),
                     passwd=config(label, 'pass'), db=config(label, 'db'))
    else:
        _dict = dict(host=host, port=port, user=user, passwd=password, db=db)

    db_info = {
        'host': _dict['host'],
        'port': _dict['port'],
        'user': _dict['user'],
        'password': _dict['passwd'],
        'charset': 'utf8mb4',
    }

    if con_type == 'get_con':
        _add_param = {}
    elif con_type == 'get_tuple':
        _add_param = {'database': _dict['db']}
    else:
        _add_param = {'database': _dict['db'], 'cursorclass': pymysql.cursors.DictCursor, }
    db_info.update(_add_param)
    # print(db_info)
    if is_multi_thread:
        poolDB = PooledDB(
            creator=pymysql,  # 指定数据库连接驱动
            maxconnections=maxconnections,  # 连接池允许的最大连接数,0和None表示没有限制
            mincached=mincached,  # 初始化时,连接池至少创建的空闲连接,0表示不创建
            maxcached=maxcached,  # 连接池中空闲的最多连接数,0和None表示没有限制
            maxshared=maxshared,  # 连接池中最多共享的连接数量,0和None表示全部共享(其实没什么卵用)
            blocking=True,  # 连接池中如果没有可用共享连接后,是否阻塞等待,True表示等等,False表示不等待然后报错
            setsession=[],  # 开始会话前执行的命令列表
            ping=0,  # ping Mysql服务器检查服务是否可用
            connect_timeout=300,
            **db_info
        )
        return poolDB
    else:
        _state = True
        _max_retries_count = 10  # 设置最大重试次数
        _conn_retries_count = 0  # 初始重试次数
        while _state and _conn_retries_count < _max_retries_count:
            try:
                poolDB = PersistentDB(
                    creator=pymysql,  # 指定数据库连接驱动
                    maxusage=maxusage,  # 一个连接最大复用次数,0或者None表示没有限制,默认为0
                    connect_timeout=300,
                    **db_info
                )
                return poolDB
            except Exception as e:
                _conn_retries_count += 1
                time.sleep(1)
                if _conn_retries_count == _max_retries_count:
                    raise e


def _get_update_key(update_df, old_df, unique_key):
    update_list = []
    for i in range(len(update_df)):
        row_dict = update_df.iloc[i].dropna().to_dict()
        key = row_dict[unique_key]
        old_data = old_df[old_df[unique_key] == key].iloc[0].dropna().to_dict()
        tuple_set = set(row_dict.items()) - set(old_data.items())
        # print(tuple_set)
        if tuple_set:
            dict_update = {}
            dict_update[unique_key] = old_data[unique_key]
            for tuple_one in tuple_set:
                dict_update[tuple_one[0]] = tuple_one[1]
            update_list.append(dict_update)
    return update_list


class DB(object):
    def __init__(self, label, **kwargs):
        """
        :param label:
        :param kwargs: con_type='get_tuple'/'get_dict'
        :param is_multi_thread: is_multi_thread default False
        :param maxconnections:
        :param mincached:
        :param maxcached:
        :param maxshared:
        :param maxusage:
        """
        self.is_multi_thread = kwargs.get('is_multi_thread', False)
        self.engine = engine_con(label, **kwargs)
        self.db_pool = get_db_con(label, **kwargs)

    # mysql数据库操作
    def operate_sql(self, sql):
        con = self.db_pool.connection()
        try:
            with con.cursor() as cursor:
                cursor.execute(sql)
            con.commit()
        except Exception as e:
            con.rollback()
            raise e
        finally:
            con.close()

    # 主要用来insert into 多条数据
    def operate_execute_many(self, sql, data):
        con = self.db_pool.connection()
        try:
            with con.cursor() as cursor:
                cursor.executemany(sql, data)
            con.commit()
        except Exception as e:
            con.rollback()
            raise e
        finally:
            con.close()

    # 可根据con_type='get_tuple'/'get_dict'调整获取是列表字典还是元组
    def get_data_sql(self, sql):
        con = self.db_pool.connection()
        try:
            with con.cursor() as cursor:
                cursor.execute(sql)
                results = cursor.fetchall()
                return results
        except Exception as e:
            raise e
        finally:
            con.close()

    def get_df_sql(self, sql):
        df = pd.read_sql(sql, self.engine, coerce_float=False)
        return df

    def get_df_sql_loop(self, sql, num):
        df = pd.read_sql(sql, self.engine, coerce_float=False, chunksize=num)
        return df

    # write current_labeled to mysql.
    def to_sql_write_data(self, data, table_name):
        if isinstance(data, list):
            df = pd.DataFrame(data)
        else:
            df = data
        df.to_sql(table_name, self.engine, if_exists='append', index=False, chunksize=5000)

    # 此函数中df的columns或dict的key值必须和数据库中的字段一致
    def execute_many_write_data(self, data, table_name):
        if not isinstance(data, list):
            list_dict = data.to_dict('records')
        else:
            list_dict = data
        list_dict = DataTypeToStr.list_dict_values_type_to_str(list_dict)
        _columns_list = list_dict[0].keys()
        _columns_num = len(_columns_list)
        _columns = '`,`'.join(_columns_list)
        _columns = '`' + _columns + '`'
        _placeholder = ','.join(['%s'] * _columns_num)
        _sql = 'insert ignore into `%s` (' % table_name + _columns + ') values (' + _placeholder + ')'
        _list_tuple = [tuple(_dict.values()) for _dict in list_dict]
        self.operate_execute_many(_sql, _list_tuple)

    def update_table_data(self, list_dict, table_name, unique_key):
        _update_dict = {'insert': [], 'update': []}
        if isinstance(list_dict, list):
            origin_df = pd.DataFrame(list_dict)
        else:
            origin_df = list_dict

        unique_key_df = self.get_df_sql("""select {0} from {1}""".format(unique_key, table_name)).dropna()

        if len(unique_key_df) != 0:
            unique_key_list = unique_key_df[unique_key].tolist()
            append_id_list = list(set(origin_df[unique_key].tolist()) - set(unique_key_list))
            if append_id_list:
                append_df = origin_df[origin_df[unique_key].isin(append_id_list)]
                append_df = append_df.drop_duplicates(unique_key)
                _update_dict['insert'] = append_df.to_dict('records')
                self.to_sql_write_data(append_df, table_name)
                # self.execute_many_write_data(append_df.astype(str), table_name)
                print(table_name + '表中添加了{}条新记录。'.format(len(append_df)))
            else:
                print(table_name + '表中没有新数据添加。')

            update_id_list = list(set(origin_df[unique_key].tolist()) - set(append_id_list))
            if update_id_list:
                columns_str = ','.join(origin_df.columns.tolist())
                if len(update_id_list) != 1:
                    _where = " from {0} where {1} in {2}".format(table_name, unique_key, tuple(update_id_list))
                else:
                    _where = " from {0} where {1}='{2}'""".format(table_name, unique_key, update_id_list[0])
                _sql = "select " + columns_str + _where
                if isinstance(list_dict, list):
                    old_data = self.get_data_sql(_sql)
                    old_df = pd.DataFrame(old_data)
                else:
                    old_df = self.get_df_sql(_sql)

                update_df = origin_df[origin_df[unique_key].isin(update_id_list)]
                update_df = update_df.drop_duplicates(unique_key)
                # print('match_start', datetime.datetime.now())
                loop_num = int(len(update_df) / _ProcessNum) + 1
                pool = Pool(processes=_ProcessNum)
                jobs = []
                for i in range(0, len(update_df), loop_num):
                    p = pool.apply_async(_get_update_key, (update_df.iloc[i:i + loop_num], old_df, unique_key,))
                    jobs.append(p)
                pool.close()  # 关闭进程池，表示不能在往进程池中添加进程
                pool.join()  # 等待进程池中的所有进程执行完毕，必须在close()之后调用

                update_data = []
                for j in jobs:
                    update_data = update_data + j.get()

                # print(update_data)
                # print('match_end', datetime.datetime.now())
                if update_data:
                    _update_dict['update'] = update_data
                    for data_dict in update_data:
                        _sql_update = "update {0} set ".format(table_name, ) + \
                                      ','.join(
                                          ['%s=%r' % (k, str(data_dict[k])) for k in data_dict if k != unique_key]) + \
                                      " where {0}='{1}'".format(unique_key, data_dict[unique_key])
                        # print(_sql_update)
                        self.operate_sql(_sql_update)
                    print(table_name + '更新了{0}条记录。'.format(len(update_data)))
                else:
                    print(table_name + '表中数据无更新。')
        else:
            _update_dict['insert'] = origin_df.to_dict('records')
            write_df = origin_df.drop_duplicates(unique_key)
            self.to_sql_write_data(write_df, table_name)
        return _update_dict

    def dispose(self):
        if self.is_multi_thread:
            self.db_pool.close()
        self.engine.dispose()


if __name__ == '__main__':
    _db = DB('160')
    # res = _db.get_data_sql("select * from risk_push_logs limit 1")
    df = _db.get_df_sql("""select model_class,report_name,row_name from report_model limit 10""")
    _db.operate_sql("truncate table test_table")
    _db.execute_many_write_data(df, 'test_table')
    # _db.dispose()
    # print(res)
    print(df)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''                                                                                                             
Author: xuwei                                        
Email: 18810079020@163.com                                 
File: test_multi_process_thread.py
Date: 2020/4/28 12:05 下午
'''

import time
from general_tools.multi_process_thread import multi_process_map, multi_thread_map, multi_thread_return, multi_process_by_pool


def fun(a):
    time.sleep(a[-1])
    print(sum(a))
    return sum(a)


if __name__ == '__main__':
    pass
    start = time.time()
    data = multi_thread_return(fun, ([0, 2], [2, 3], [3, 4]), thread_num=3, timeout=1)
    end = time.time()
    print(end - start)
    print(data)


    start = time.time()
    # data = multi_process_map(fun, ([0, 1], [2, 3], [4]), pool_num=3, if_return=True)
    data = multi_process_by_pool(fun, ([0, 2], [2, 3], [3, 4]), pool_num=3, timeout=1)
    # data = multi_thread_map(fun, ([0, 2], [2, 3], [4]), 3, if_return=True, timeout=1)
    # data = multi_thread_return(fun, ([0, 2], [2, 3], [3, 4]), 6, timeout=0.2)
    end = time.time()
    print(end - start)
    print(data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Author: xuwei
Email: 18810079020@163.com
Description:
date:
'''

import pymongo
from pymongo import UpdateOne
from .config_log import config


def get_client(label='mongo', **kwargs):
    if config(label, 'user', ''):
        my_client = pymongo.MongoClient(
            'mongodb://{0}:{1}@{2}:{3}/'. \
                format(
                config(label, 'user'), config(label, 'pass'),
                config(label, 'host'), config(label, 'port')
            ),
            **kwargs
        )
    else:
        my_client = pymongo.MongoClient(
            'mongodb://{0}:{1}/'. \
                format(config(label, 'host'), config(label, 'port')),
            **kwargs
        )
    return my_client


class MongoOp(object):
    def __init__(self, db, collection, label='mongo', limit=10):
        self.db = db
        self.limit = limit
        self.client = get_client(label=label)
        self.col = self.client[db][collection]

    def find_count(self, *args, **kwargs):
        return self.col.find(*args, **kwargs).count()

    def show_db(self):
        dbs_list = self.client.database_names()
        return dbs_list

    def show_col(self):
        db = self.client[self.db]
        cols_list = db.collection_names()
        return cols_list

    # create_index 只会在索引不存在的时候创建一个索引（ensure_index是老版本的）
    def create_index(self, keys, session=None, **kwargs):
        """
        :param keys: [('name', pymongo.ASCENDING), ('create_time', pymongo.ASCENDING)]
        :param session:
        :param kwargs: create_time=1, expireAfterSeconds=3600*20
        :return:
        """
        self.col.create_index(keys, session=session, **kwargs)

    def drop_index(self, index_or_name, session=None, **kwargs):
        self.col.drop_index(index_or_name, session=session, **kwargs)

    def index_info(self):
        return self.col.index_information()

    def view_index(self):
        indexs_list = self.col.list_indexes()
        for index in indexs_list:
            print(index)

    def drop_col(self):
        self.col.drop()

    def batch_write(self, write_list_dict):
        to_be_request = [UpdateOne({'_id': b['_id']}, {'$set': b}, upsert=True) for b in write_list_dict]
        self.col.bulk_write(to_be_request, ordered=False)

    def insert_one(self, write_dict):
        """
        :param write_dict:
        :return: 返回col中这条数据对象
        """
        try:
            return self.col.insert_one(write_dict)
        except:
            pass

    def insert(self, write_dict):
        """
        :param write_dict:
        :return: 返回col中这条数据的id
        """
        try:
            return self.col.insert(write_dict)
        except:
            pass

    def find_one(self, *args, **kwargs):
        res = self.col.find_one(*args, **kwargs)
        return res

    def find(self, *args, **kwargs):
        yield_res = self.col.find(*args, **kwargs).limit(self.limit)
        res_list = [_ for _ in yield_res]
        return res_list

    def find_all_by_iter(self, *args, limit=None, **kwargs):
        if limit:
            yield_res = self.col.find(*args, **kwargs).limit(self.limit)
        else:
            yield_res = self.col.find(*args, **kwargs)
        return yield_res

    def close(self):
        self.client.close()


# 设置过期时间示例
def message_more_cache_expire_index():
    mg_op = MongoOp('xiaomu', 'message_more_cache', label='mongo')
    mg_op.drop_index('time_-1')
    mg_op.create_index([("time", pymongo.DESCENDING)], expireAfterSeconds=3600*18)

if __name__ == '__main__':
    mg_op = MongoOp('xiaomu', 'knowledge_map', label='mongo')
    # mg_op.create_index([("other_name", pymongo.ASCENDING), ("stock_code", pymongo.ASCENDING)])
    r = mg_op.show_col()
    print(r)
    mg_op.view_index()
    r = mg_op.find_one({'type': 'teacher'})
    print(r)
    r = mg_op.find({'type': 'teacher'})
    print(r)
    mg_op.close()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
File: {name}.py
Author: xuwei
Email: weix@sensedeal.ai
Last modified: 2018.12.23
Description:
'''

import json
import numpy
import pymysql
import datetime
import pandas as pd
from datetime import date
from .config_log import config
from multiprocessing import Pool
from sqlalchemy import create_engine

# import warnings
# warnings.filterwarnings("ignore")
# from common.config_log import config

_ProcessNum = 3
_Thread = 4


class DataTypeToStr(object):
    @classmethod
    def data_type_to_str(cls, obj):
        if obj != obj:
            return None
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, numpy.datetime64):
            return pd.to_datetime(str(obj)).strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, numpy.integer) or isinstance(obj, numpy.floating) or isinstance(obj, int) \
                or isinstance(obj, float) or isinstance(obj, bytes):
            return str(obj)
        elif isinstance(obj, numpy.ndarray):
            return json.dumps(obj.tolist(), ensure_ascii=False)
        elif isinstance(obj, list):
            return json.dumps(obj, ensure_ascii=False)
        elif isinstance(obj, dict):
            return json.dumps(obj, ensure_ascii=False)
        else:
            return obj

    @classmethod
    def dict_values_type_etl_str(cls, dict_param):
        for key in dict_param:
            dict_param[key] = cls.data_type_to_str(dict_param[key])
        return dict_param

    @classmethod
    def list_dict_values_type_to_str(cls, list_dict):
        list_dict = list(map(cls.dict_values_type_etl_str, list_dict))
        return list_dict


class MyException(Exception):  # 让MyException类继承Exception
    def __init__(self, str_describe):
        self.msg = str_describe


class ConMysql(object):
    def __init__(self, label=None, single_entity=False, host=None, port=None, user=None, password=None, db=None):
        self._single_entity = single_entity
        self.__con_entity = dict()
        if label:
            self._host = config(label, 'host')
            self._port = int(config(label, 'port'))
            self._user = config(label, 'user')
            self._password = config(label, 'pass')
            self._db_name = db if db else config(label, 'db')
        else:
            self._host = host
            self._port = port
            self._user = user
            self._password = password
            self._db_name = db

    def __get_engine_con(self):
        _state = True
        _max_retries_count = 10  # 设置最大重试次数
        _conn_retries_count = 0  # 初始重试次数
        while _state and _conn_retries_count < _max_retries_count:
            try:
                engine = create_engine("mysql+pymysql://{user}:{passwd}@{host}:{port}/{db}".format(
                    host=self._host,
                    port=self._port,
                    user=self._user,
                    passwd=self._password,
                    db=self._db_name,
                ), connect_args={'charset': 'utf8'},
                    pool_size=5,
                    pool_recycle=1800,
                )
                return engine
            except:
                _conn_retries_count += 1
        raise MyException('%s-engine_db数据库连接失败' % self._host)

    def __get_con_db_dict(self):
        _state = True
        _max_retries_count = 10  # 设置最大重试次数
        _conn_retries_count = 0  # 初始重试次数
        while _state and _conn_retries_count < _max_retries_count:
            try:
                con = pymysql.connect(
                    host=self._host,
                    port=self._port,
                    user=self._user,
                    passwd=self._password,
                    db=self._db_name,
                    charset='utf8',
                    cursorclass=pymysql.cursors.DictCursor,
                )
                return con
            except:
                _conn_retries_count += 1
        raise MyException('%s-con_db_dict数据库连接失败' % self._host)

    def __get_con_db(self):
        _state = True
        _max_retries_count = 10  # 设置最大重试次数
        _conn_retries_count = 0  # 初始重试次数
        while _state and _conn_retries_count < _max_retries_count:
            try:
                con = pymysql.connect(
                    host=self._host,
                    port=self._port,
                    user=self._user,
                    passwd=self._password,
                    db=self._db_name,
                    charset='utf8mb4',
                )
                return con
            except:
                _conn_retries_count += 1
        raise MyException('%s-con_db数据库连接失败' % self._host)

    def engine_db(self):
        if self._single_entity:
            if 'engine_db' in self.__con_entity:
                return self.__con_entity['engine_db']
            else:
                self.__con_entity['engine_db'] = self.__get_engine_con()
                return self.__con_entity['engine_db']
        else:
            return self.__get_engine_con()

    # 数据形式为列表字典
    def con_db_dict(self):
        if self._single_entity:
            if 'con_db_dict' in self.__con_entity:
                try:
                    self.__con_entity['con_db_dict'].ping()
                    return self.__con_entity['con_db_dict']
                except:
                    self.__con_entity['con_db_dict'] = self.__get_con_db_dict()
                    return self.__con_entity['con_db_dict']
            else:
                self.__con_entity['con_db_dict'] = self.__get_con_db_dict()
                return self.__con_entity['con_db_dict']
        else:
            return self.__get_con_db_dict()

    # 数据形式为元组
    def con_db(self):
        if self._single_entity:
            if 'con_db' in self.__con_entity:
                try:
                    self.__con_entity['con_db'].ping()
                    return self.__con_entity['con_db']
                except:
                    self.__con_entity['con_db'] = self.__get_con_db()
                    return self.__con_entity['con_db']
            else:
                self.__con_entity['con_db'] = self.__get_con_db()
                return self.__con_entity['con_db']
        else:
            return self.__get_con_db()

    def close_all(self):
        if self._single_entity:
            if self.__con_entity.get('engine_db'):
                self.__con_entity['engine_db'].dispose()
            if self.__con_entity.get('con_db'):
                self.__con_entity['con_db'].close()
            if self.__con_entity.get('con_db_dict'):
                self.__con_entity['con_db_dict'].close()


class SqlDateOp(ConMysql):
    # mysql数据库操作
    def operate_sql(self, sql, close_con=True):
        con = self.con_db()
        try:
            with con.cursor() as cursor:
                cursor.execute(sql)
            con.commit()
        except Exception as e:
            con.rollback()
            raise e
        finally:
            if close_con:
                con.close()

    def get_dict_sql(self, sql, close_con=True):
        con = self.con_db_dict()
        try:
            with con.cursor() as cursor:
                cursor.execute(sql)
                results = cursor.fetchall()
                return results
        except Exception as e:
            raise e
        finally:
            if close_con:
                con.close()

    def get_tuple_sql(self, sql, close_con=True):
        con = self.con_db()
        try:
            with con.cursor() as cursor:
                cursor.execute(sql)
                results = cursor.fetchall()
                return results
        except Exception as e:
            raise e
        finally:
            if close_con:
                con.close()

    # 主要用来insert into 多条数据
    def operate_execute_many(self, sql, data, close_con=True):
        con = self.con_db()
        try:
            with con.cursor() as cursor:
                cursor.executemany(sql, data)
            con.commit()
        except Exception as e:
            con.rollback()
            raise e
        finally:
            if close_con:
                con.close()

    # # 此函数中df的columns或dict的key值必须和数据库中的字段一致
    # def execute_many_write_data(self, data, table_name):
    #     if isinstance(data, list):
    #         df = pd.DataFrame(data)
    #     else:
    #         df = data
    #     _columns_list = df.columns
    #     _columns_num = len(_columns_list)
    #     _columns = '`,`'.join(_columns_list)
    #     _columns = '`' + _columns + '`'
    #     _placeholder = ','.join(['%s'] * _columns_num)
    #     _sql = 'insert ignore into `%s` (' % table_name + _columns + ') values (' + _placeholder + ')'
    #     _list_tuple = [tuple(obj) for obj in df.to_records(index=False)]
    #     self.operate_execute_many(_sql, _list_tuple)

    # 此函数中df的columns或dict的key值必须和数据库中的字段一致
    def execute_many_write_data(self, data, table_name, close_con=False):
        """
        :param data: df or list_dict
        :param table_name:
        :param close_con:
        :return:
        """
        if not isinstance(data, list):
            list_dict = data.to_dict('records')
        else:
            list_dict = data
        list_dict = DataTypeToStr.list_dict_values_type_to_str(list_dict)
        _columns_list = list_dict[0].keys()
        _columns_num = len(_columns_list)
        _columns = '`,`'.join(_columns_list)
        _columns = '`' + _columns + '`'
        _placeholder = ','.join(['%s'] * _columns_num)
        _sql = 'insert ignore into `%s` (' % table_name + _columns + ') values (' + _placeholder + ')'
        _list_tuple = [tuple(_dict.values()) for _dict in list_dict]
        self.operate_execute_many(_sql, _list_tuple, close_con=close_con)

    def get_df_sql(self, sql):
        try:
            df = pd.read_sql(sql, self.engine_db(), coerce_float=False)
            return df
        except Exception as e:
            raise e
        finally:
            self.engine_db().dispose()

    def get_df_sql_loop(self, sql, num):
        try:
            df = pd.read_sql(sql, self.engine_db(), coerce_float=False, chunksize=num)
            return df
        except Exception as e:
            raise e
        finally:
            self.engine_db().dispose()

    # write current_labeled to mysql.
    def to_sql_write_data(self, data, table_name, if_exists='append'):
        try:
            if isinstance(data, list):
                df = pd.DataFrame(data)
            else:
                df = data
            df.to_sql(table_name, self.engine_db(), if_exists=if_exists, index=False, chunksize=10000)
        except Exception as e:
            raise e
        finally:
            self.engine_db().dispose()


def _get_update_key(update_df, old_df, unique_key):
    update_list = []
    for i in range(len(update_df)):
        row_dict = update_df.iloc[i].dropna().to_dict()
        key = row_dict[unique_key]
        old_data = old_df[old_df[unique_key] == key].iloc[0].dropna().to_dict()
        tuple_set = set(row_dict.items()) - set(old_data.items())
        # print(tuple_set)
        if tuple_set:
            dict_update = {}
            dict_update[unique_key] = old_data[unique_key]
            for tuple_one in tuple_set:
                dict_update[tuple_one[0]] = tuple_one[1]
            update_list.append(dict_update)
    return update_list


class UpdateTable(SqlDateOp):
    def update_table_data(self, list_dict, table_name, unique_key):
        _update_dict = {'insert': [], 'update': []}
        if isinstance(list_dict, list):
            origin_df = pd.DataFrame(list_dict)
        else:
            origin_df = list_dict

        unique_key_df = self.get_df_sql("""select {0} from {1}""".format(unique_key, table_name)).dropna()

        if len(unique_key_df) != 0:
            unique_key_list = unique_key_df[unique_key].tolist()
            append_id_list = list(set(origin_df[unique_key].tolist()) - set(unique_key_list))
            if append_id_list:
                append_df = origin_df[origin_df[unique_key].isin(append_id_list)]
                append_df = append_df.drop_duplicates(unique_key)
                _update_dict['insert'] = append_df.to_dict('records')
                self.to_sql_write_data(append_df, table_name)
                # self.execute_many_write_data(append_df.astype(str), table_name)
                print(table_name + '表中添加了{}条新记录。'.format(len(append_df)))
            else:
                print(table_name + '表中没有新数据添加。')

            update_id_list = list(set(origin_df[unique_key].tolist()) - set(append_id_list))
            if update_id_list:
                columns_str = ','.join(origin_df.columns.tolist())
                if len(update_id_list) != 1:
                    _where = " from {0} where {1} in {2}".format(table_name, unique_key, tuple(update_id_list))
                else:
                    _where = " from {0} where {1}='{2}'""".format(table_name, unique_key, update_id_list[0])
                _sql = "select " + columns_str + _where
                if isinstance(list_dict, list):
                    old_data = self.get_dict_sql(_sql)
                    old_df = pd.DataFrame(old_data)
                else:
                    old_df = self.get_df_sql(_sql)

                update_df = origin_df[origin_df[unique_key].isin(update_id_list)]
                update_df = update_df.drop_duplicates(unique_key)
                # print('match_start', datetime.datetime.now())
                loop_num = int(len(update_df) / _ProcessNum) + 1
                pool = Pool(processes=_ProcessNum)
                jobs = []
                for i in range(0, len(update_df), loop_num):
                    p = pool.apply_async(_get_update_key, (update_df.iloc[i:i + loop_num], old_df, unique_key,))
                    jobs.append(p)
                pool.close()  # 关闭进程池，表示不能在往进程池中添加进程
                pool.join()  # 等待进程池中的所有进程执行完毕，必须在close()之后调用

                update_data = []
                for j in jobs:
                    update_data = update_data + j.get()

                # print(update_data)
                # print('match_end', datetime.datetime.now())
                if update_data:
                    _update_dict['update'] = update_data
                    for data_dict in update_data:
                        _sql_update = "update {0} set ".format(table_name, ) + \
                                      ','.join(
                                          ['%s=%r' % (k, str(data_dict[k])) for k in data_dict if k != unique_key]) + \
                                      " where {0}='{1}'".format(unique_key, data_dict[unique_key])
                        # print(_sql_update)
                        self.operate_sql(_sql_update)
                    print(table_name + '更新了{0}条记录。'.format(len(update_data)))
                else:
                    print(table_name + '表中数据无更新。')
        else:
            _update_dict['insert'] = origin_df.to_dict('records')
            write_df = origin_df.drop_duplicates(unique_key)
            self.to_sql_write_data(write_df, table_name)
        return _update_dict


if __name__ == '__main__':
    _db = SqlDateOp('160', 'xw_test', single_entity=True)
    # _db_1 = SqlDateOp('160', 'xw_test', single_entity=True)
    # print(id(_db))
    # print(id(_db_1))
    df = _db.get_dict_sql("""select * from test_table limit 10""", close_con=False)
    print(df)
    df = _db.get_dict_sql("""select * from test_table limit 10""", close_con=False)
    print(df)
    _db.close_all()



from .queue0 import *
from .log import log_exception, log_info, log_warn
from .utils import sleep
from pika.exceptions import *


class RabbitProducer(RabbitProducer0):

    def send_safely(self, topic, message, need_close=False, times=None, **kwargs):
        if times is None:
            times = 4
        while times >= 0:
            try:
                times -= 1
                self.send(topic, message, need_close, **kwargs)
                return True
            except Exception as ex:
                log_warn("send message fir {0} failed ex:{1}".format(topic, str(ex)))
                self.close_safely()
                sleep(1)
        return False

    def close_safely(self):
        try:
            self.close_connection()
        except Exception as ex:
            log_exception(str(ex))


class RabbitConsumer(RabbitConsumer0):

    def _close_safely(self):
        try:
            self.close_connection()
        except Exception as ex:
            log_exception(ex)

    def execute_safely(self, caller, prefetch_count=1, no_ack=False, exchange=None):
        while True:
            try:
                self.execute(caller, prefetch_count, no_ack, exchange)
            except (ConnectionClosed, ChannelClosed, ConnectionError) as ex:
                log_info("rabbit except {0}".format(ex))
                self.close_connection()
                sleep(2)
            except Exception as ex:
                log_exception(ex)
                self._close_safely()
                raise ex


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
Author: xuwei
Email: weix@sensedeal.ai
Description:
'''

import configparser
import logging
import sys
import os

__local_config = 'settings.local.ini'

__server_config = 'settings.ini'

session = 'settings'

log_path = ''

server_module = ''

format_log = False

__root_deep = 8

SETTINGS_PATH = ''


class MyException(Exception):  # 让MyException类继承Exception
    def __init__(self, str_describe):
        self.msg = str_describe


def config_path(cur_path):
    filename_local = os.path.join(cur_path, __local_config)
    filename = os.path.join(cur_path, __server_config)
    if os.path.isfile(filename_local):
        return filename_local, True
    elif os.path.isfile(filename):
        return filename, True
    else:
        up_path = os.path.dirname(cur_path)
        return up_path, False


def init_config_path():
    global SETTINGS_PATH
    if SETTINGS_PATH: return
    root_deep = __root_deep
    current_path = sys.path[0]
    while root_deep > 0:
        file_path, state = config_path(current_path)
        if state:
            SETTINGS_PATH = file_path
            return None
        else:
            current_path = os.path.dirname(current_path)
            root_deep -= 1
    print('not find settings config file')


class Config(object):
    def __init__(self):
        init_config_path()
        self.cf = configparser.ConfigParser()
        self.cf.read(SETTINGS_PATH)

    def __call__(self, *args):
        try:
            if len(args) == 1:
                parameter = self.cf.get(session, args[0])
            else:
                parameter = self.cf.get(*args[:2])
            return parameter
        except Exception as e:
            if len(args) > 2:
                return args[2]
            raise e



config = Config()


def init_log(file_path='', module='', log_format=False):
    if not os.path.exists(file_path):
        os.mkdir(file_path)
        # raise MyException('log_path %s not exist.' % file_path)
    global log_path, server_module, format_log
    log_path = file_path
    if module:
        server_module = module + '.'
    format_log = log_format


class Log(object):
    def __init__(self):
        self.logger = logging.getLogger()
        self.logger.setLevel(logging.DEBUG)
        # 日志输出格式
        # self.formatter = logging.Formatter('[%(asctime)s] - %(filename)s] - %(levelname)s: %(message)s')
        # self.formatter = logging.Formatter('[%(asctime)s - %(levelname)s] - %(message)s')
        self.formatter = logging.Formatter('[%(levelname)s][%(asctime)s]%(message)s')

    def __console(self, level, message):
        if not log_path: return
        # 创建一个FileHandler，用于写到本地
        if level == 'info':
            log_name = os.path.join(log_path, '%sinfo.log' % server_module)
        elif level == 'debug':
            log_name = os.path.join(log_path, '%sdebug.log' % server_module)
        elif level == 'warning':
            log_name = os.path.join(log_path, '%swarning.log' % server_module)
        else:
            log_name = os.path.join(log_path, '%serror.log' % server_module)

        fh = logging.FileHandler(log_name, 'a', encoding='utf-8')  # 这个是python3的
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(self.formatter)
        self.logger.addHandler(fh)

        # # 创建一个StreamHandler,用于输出到控制台
        # ch = logging.StreamHandler()
        # ch.setLevel(logging.DEBUG)
        # ch.setFormatter(self.formatter)
        # self.logger.addHandler(ch)
        data = message
        if server_module and format_log:
            data = {'module': server_module, 'log_level': level, 'msg': message}
        if level == 'info':
            self.logger.info(data)
        elif level == 'debug':
            self.logger.debug(data)
        elif level == 'warning':
            self.logger.warning(data)
        elif level == 'error':
            self.logger.error(data)
        elif level == 'except':
            self.logger.exception(data)

        # 这两行代码是为了避免日志输出重复问题
        # self.logger.removeHandler(ch)
        self.logger.removeHandler(fh)
        # 关闭打开的文件
        fh.close()

    def debug(self, message):
        self.__console('debug', message)

    def info(self, message):
        self.__console('info', message)

    def warning(self, message):
        self.__console('warning', message)

    def error(self, message):
        self.__console('error', message)

    def exception(self, message):
        self.__console('except', message)

    def raise_exception(self, message):
        self.exception(message)
        raise Exception(message)



log = Log()

if __name__ == '__main__':
    host = config('db_stock', 'host', '12')
    print(host)
    # port = config('log_path')
    # print(port)
    # init_log(module='test', file_path=config('log_path'), log_format=True)
    # log.info('this is a test1')
    # log.error('this is a test2')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C)2018 SenseDeal AI, Inc. All Rights Reserved
File: .py
Author: xuwei
Email: weix@sensedeal.ai
Last modified: date
Description:
'''

from .config_log import config
import json
import redis
import time

__client_map = {}

def get_redis_con(label='redis', db=None, config_map=None):
    global __client_map
    label1 = label + str(db)
    if label1 not in __client_map:
        if config_map:
            redis_host = config_map['host']
            redis_port = int(config_map['port'])
            redis_pass = config_map['pass'] if config_map.get('pass') else config_map['password']
        else:
            redis_host = config(label, 'host')
            redis_port = int(config(label, 'port'))
            redis_pass = config(label, 'pass', '')
            if redis_pass == '':
                redis_pass = config(label, 'password', '')
        if len(redis_pass) == 0:
            redis_pass = None
        redis_pool = redis.ConnectionPool(host=redis_host, port=redis_port, password=redis_pass, db=db)
        __client_map[label1] = redis_pool
    clients = redis.Redis(connection_pool=__client_map[label1])
    return clients

class RedisOperate(object):
    def __init__(self, label='redis', db=None):
        self.label = label
        self.db = db

    def redis_hkeys(self, name):
        get_con = get_redis_con(label=self.label, db=self.db)
        keys = get_con.hkeys(name)
        return keys

    def get(self, name):
        get_con = get_redis_con(label=self.label, db=self.db)
        keys = get_con.get(name)
        return keys

    def set(self, name, value):
        get_con = get_redis_con(label=self.label, db=self.db)
        keys = get_con.set(name, value)
        return keys

    def redis_hget(self, name, key):
        get_con = get_redis_con(label=self.label, db=self.db)
        val = get_con.hget(name, key)
        return val

    def redis_hgetall(self, name):
        get_con = get_redis_con(label=self.label, db=self.db)
        res_dict = get_con.hgetall(name)
        return res_dict

    # 列表字典
    def redis_one_dict_hset(self, name, one_dict, key):
        _num = 0
        while _num < 4:
            try:
                get_con = get_redis_con(label=self.label, db=self.db)
                get_con.hset(name, one_dict[key], json.dumps(one_dict, ensure_ascii=False))
                return
            except:
                _num += 1
                time.sleep(1)

    # 列表字典
    def other_name_to_redis(self, name, list_dict, key):
        try:
            get_con = get_redis_con(label=self.label, db=self.db)
            write_pip = get_con.pipeline()
            for one_dict in list_dict:
                plate_value_dict = json.loads(one_dict['plate_value'])
                one_dict['plate'] = plate_value_dict['plate']
                one_dict['plate_value'] = plate_value_dict['plate_value']
                write_pip.hset(name, one_dict[key], json.dumps(one_dict, ensure_ascii=False))
            write_pip.execute()
        except Exception as e:
            print(e)

    def redis_hset(self, name, key, value):
        get_con = get_redis_con(label=self.label, db=self.db)
        get_con.hset(name, key, value)

    # [(key1,value1),(key2,value2)]
    def redis_hset_tuple(self, name, list_tuple):
        get_con = get_redis_con(label=self.label, db=self.db)
        write_pip = get_con.pipeline()
        for one_tuple in list_tuple:
            write_pip.hset(name, one_tuple[0], one_tuple[1])
        write_pip.execute()

    def redis_lpush(self, name, list_data):
        get_redis_con(label=self.label, db=self.db).lpush(name, json.dumps(list_data))

    def redis_expire(self, name, time_pram):
        get_redis_con(label=self.label, db=self.db).expire(name, time_pram)

    def redis_llen(self, name):
        return get_redis_con(label=self.label, db=self.db).llen(name)

    def redis_lrange(self, name, start, end):
        return get_redis_con(label=self.label, db=self.db).lrange(name, start, end)

    def redis_hdel(self, name, *keys):
        return get_redis_con(label=self.label, db=self.db).hdel(name, *keys)


if __name__ == '__main__':
    xw_redis = RedisOperate('redis')
    keys = xw_redis.redis_hkeys('stock_company')
    val = xw_redis.redis_hget('stock_company', '10181879')
    res_dict = xw_redis.redis_hgetall('stock_company')

    # print(keys)
    # print(val)
    print(res_dict)
